'''
FilePath: /python-Sigilyph/sigilyph/core/preprocess.py
Descripttion: 
Author: Yixiang Chen
version: 
Date: 2025-05-13 11:01:26
LastEditors: Yixiang Chen
LastEditTime: 2025-05-14 20:26:27
'''

def replace_proper(text, namedict):
    for k,v in namedict.items():
        text = text.replace(k,v)
    return text





'''
FilePath: /python-Sigilyph/sigilyph/core/g2p_func.py
Descripttion: 
Author: Yixiang Chen
version: 
Date: 2025-03-31 16:55:51
LastEditors: Yixiang Chen
LastEditTime: 2025-08-12 14:42:02
'''

from g2p_en import G2p
_g2p_en = G2p()

def g2p_en(text, sp_sign='<sp>'):
    phone_list = _g2p_en(text)
    phone_list = [sp_sign if xx == " " else xx for xx in phone_list]
    if len(phone_list)>1 and phone_list[-1] != sp_sign:
        phone_list.append(sp_sign) 
    return phone_list


from pypinyin import lazy_pinyin, Style

from sigilyph.core.symbols import punctuation, punc_map_ch, cn_word2phone_dict
for punc in punctuation:
    cn_word2phone_dict[punc] = punc

def g2p_cn(text):
    pinyinlist = lazy_pinyin(text, style=Style.TONE3, neutral_tone_with_five=True, tone_sandhi=True)
    outlist = []
    for pp in pinyinlist:
        if pp in cn_word2phone_dict.keys():
            outlist.extend(cn_word2phone_dict[pp])
            outlist.append('<sp>')
        else:
            for ch in pp:
                outlist.extend(cn_word2phone_dict[ch]) 
                outlist.append('<sp>')
    if len(outlist) > 4:
        if outlist[-2] == 'sil' and outlist[-4] == 'sil':
            outlist = outlist[:-2]
    return outlist








'''
FilePath: /python-Sigilyph/sigilyph/core/bert_align.py
Descripttion: 
Author: Yixiang Chen
version: 
Date: 2025-09-24 15:13:38
LastEditors: Yixiang Chen
LastEditTime: 2025-09-28 11:05:20
'''

import torch
from transformers import BertTokenizer, BertModel

import langid
from g2p_en import G2p
_g2p_en = G2p()

from pypinyin import lazy_pinyin, Style

from sigilyph.core.norm_func import text_norm_cn
from sigilyph.core.symbols import punctuation, punc_map_ch, cn_word2phone_dict
for punc in punctuation:
    cn_word2phone_dict[punc] = punc

def _g2p_cn(text):
    pinyinlist = lazy_pinyin(text, style=Style.TONE3, neutral_tone_with_five=True, tone_sandhi=True)
    outlist = []
    for pp in pinyinlist:
        if pp in cn_word2phone_dict.keys():
            outlist.extend(cn_word2phone_dict[pp])
        else:
            for ch in pp:
                if ch in cn_word2phone_dict.keys():
                    outlist.extend(cn_word2phone_dict[ch]) 
                else:
                    outlist.extend('sil')
    return outlist 

def g2p_word(word):
    tmp_lang = langid.classify(word)[0]
    if tmp_lang in ['zh', 'jp', 'ja']:
        tmp_lang = 'zh'
    else:
        tmp_lang = 'en' 
    if tmp_lang == 'zh':
        return _g2p_cn(word) 
    else:
        return _g2p_en(word) 

class AlignBert():
    def __init__(self, tkn_cache_dir, vocab_file):
        
        #tkn_cache_dir = "./tmp/"
        self.vocab_dict = self.load_vocab(vocab_file) 
        self.tokenizer = BertTokenizer.from_pretrained('bert-base-multilingual-cased', cache_dir=tkn_cache_dir)
        self.g2p_word = g2p_word 
        self.symbol_list = ['[CLS]', '[SEP]', '?', '!', '.', ',', '，', '。']

        self.empty_bert = torch.zeros([768])

    def gen_seqbert(self, mfa_phones, text, bert):
        norm_text = text_norm_cn(text)
        midph_list, midph2bertid_dict = self.get_midph(norm_text)
        phoneme2midph_dict = self.get_phoneme2midph_dict(mfa_phones, midph_list)
        phoneme2bertid_dict = self.get_phoneme2bertid_dict(midph2bertid_dict, phoneme2midph_dict)
        seqbert = []
        for idx in range(len(phoneme2bertid_dict)):
            bertid = phoneme2bertid_dict[idx]
            if bertid >=0:
                seqbert.append(bert[bertid])
            else:
                seqbert.append(self.empty_bert)
        seqbert = torch.stack(seqbert)
        return seqbert 

    def load_vocab(self, vocab_file):
        vocab_dict = {}
        with open(vocab_file, 'r') as ovf:
            lines = ovf.readlines()
        for idx in range(len(lines)):
            line = lines[idx]
            tt = line.strip()
            vocab_dict[idx] = tt
        del lines
        return vocab_dict

    def id2text(self, idlist):
        outlist = []
        for idx in range(len(idlist)):
            outlist.append(self.vocab_dict[int(idlist[idx])])
        return outlist
        
    def get_midph(self, text):
        encoded_input = self.tokenizer(text, return_tensors='pt')
        ret = self.id2text(encoded_input['input_ids'][0])
        wordph_list = []
        for word in ret[1:-1]:
            word_phoneme = self.g2p_word(word)
            wordph_list.append(word_phoneme)
        midph_list = []
        midph2bertid_dict = {}
        midph_list.append(ret[0])
        midph2bertid_dict[0] = 0
        for widx in range(len(wordph_list)):
            for phidx in range(len(wordph_list[widx])):
                phoneme = wordph_list[widx][phidx]
                midph_list.append(phoneme)
                midph2bertid_dict[len(midph_list)-1]=widx+1
        midph_list.append(ret[-1])
        midph2bertid_dict[len(midph_list)-1] = len(ret)-1
        return midph_list, midph2bertid_dict
    
    def get_phoneme2midph_dict(self, mfa_phones, midph_list):
        fixed_midph_list = []
        for idx in range(len(midph_list)):
            if midph_list[idx] in self.symbol_list:
                fixed_midph_list.append('sil')
            else:
                fixed_midph_list.append(midph_list[idx])
        phoneme2midph_dict = self.lcs(mfa_phones, fixed_midph_list)
        return phoneme2midph_dict

    def lcs(self, mfa_phones, midph_list):
        n, m = len(midph_list), len(mfa_phones)
        dp = [[0] * (n + 1) for _ in range(m + 1)]

        phoneme2midph_dict = {}
        for idx in range(1, m+1):
            phoneme2midph_dict[idx-1]=-1
        
        for idx in range(1, m+1):
            for midph_id in range(1, n+1):
                curr_ph = mfa_phones[idx-1]
                if curr_ph == midph_list[midph_id-1]:
                    dp[idx][midph_id] = dp[idx-1][midph_id-1] + 1
                else:
                    dp[idx][midph_id] = max(dp[idx-1][midph_id], dp[idx][midph_id-1])
        n, m = len(midph_list), len(mfa_phones)
        while m > 0 and n > 0:
            if mfa_phones[m-1] == midph_list[n-1] and dp[m][n] == dp[m-1][n-1] + 1:
                phoneme2midph_dict[m-1] = n-1
                m, n = m-1, n-1
                continue
            if dp[m][n] == dp[m-1][n]:
                m, n = m-1, n
                continue
            if dp[m][n] == dp[m][n-1]:
                m, n = m, n-1
                continue
        return phoneme2midph_dict    


    def get_phoneme2bertid_dict(self, midph2bertid_dict, phoneme2midph_dict):
        phoneme2bertid_dict = {}
        for idx in range(len(phoneme2midph_dict)):
            phoneme_id = idx
            midph_id = phoneme2midph_dict[phoneme_id] 
            #if midph_id not in midph2bertid_dict.keys():
            if midph_id == -1:
                phoneme2bertid_dict[idx] = -1
            else:
                phoneme2bertid_dict[idx] = midph2bertid_dict[midph_id]
        return phoneme2bertid_dict 



'''
FilePath: /python-Sigilyph/sigilyph/core/text_process.py
Descripttion: 
Author: Yixiang Chen
version: 
Date: 2025-03-31 16:31:26
LastEditors: Yixiang Chen
LastEditTime: 2025-08-12 14:40:42
'''


import langid
import re

import jieba
import jieba.posseg

from sigilyph.core.g2p_func import g2p_en, g2p_cn
from sigilyph.core.norm_func import preprocess_first, text_norm_en, text_norm_cn

from sigilyph.core.symbols import base_phone_set, cn_phone_set, en_phone_set, punctuation, special_phrase

#all_phone_set = [] + sorted(set(base_phone_set + cn_phone_set + en_phone_set))
#all_phone_set = [] + list(set(base_phone_set)) + list(set(cn_phone_set + en_phone_set))
all_phone_set = [] + sorted(set(base_phone_set)) + sorted(set(cn_phone_set)) + sorted(set(en_phone_set))
all_phone_dict = {xx:idx for idx, xx in enumerate(all_phone_set)}

norm_func_dict = {
    'en': text_norm_en,
    'zh': text_norm_cn
}

g2p_func_dict = {
    'en': g2p_en,
    'zh': g2p_cn
}

sil1symbol='-'

def text_split_lang_old(text, lang):
    if lang == 'ZH' or lang == 'zh':
        multi_lang_text_list = [{'lang':'zh', 'text_split': text}]
    elif lang == 'en':
        multi_lang_text_list = [{'lang':'en', 'text_split': text}]
    else:
        pattern = r'([a-zA-Z ,.\!\?\"]+|[\u4e00-\u9fa5 ，。\！\？\“\”]+)'
        text_split = re.findall(pattern, text)
        multi_lang_text_list = []
        for idx in range(len(text_split)):
            tmpts = text_split[idx]
            tmp_lang = langid.classify(tmpts)[0]
            multi_lang_text_list.append({'lang':tmp_lang, 'text_split': tmpts})
    return multi_lang_text_list

def text_split_lang_bk0724(text, lang):
    if lang == 'ZH' or lang == 'zh':
        multi_lang_text_list = [{'lang':'zh', 'text_split': text}]
    elif lang == 'en':
        multi_lang_text_list = [{'lang':'en', 'text_split': text}]
    else:
        pretext_split =  re.split("(\[.*?\])", text, re.I|re.M)
        multi_lang_text_list = []
        pretext_split = list(filter(None, pretext_split))
        for utext in pretext_split:
            if utext[0] != '[':
                pattern = r'([a-zA-Z ,.\!\?\"]+|[\u4e00-\u9fa5 ，。,.\t \！\？]+)'
                text_split = re.findall(pattern, utext)
                for idx in range(len(text_split)):
                    tmpts = text_split[idx]
                    tmp_lang = langid.classify(tmpts)[0]
                    if tmp_lang in ['zh', 'jp', 'ja']:
                        tmp_lang = 'zh'
                    else:
                        tmp_lang = 'en' 
                    if not tmpts.isspace():
                        multi_lang_text_list.append({'lang':tmp_lang, 'text_split': tmpts})
            else:
                phones = utext[1:-1]
                multi_lang_text_list.append({'lang':'phone', 'text_split': phones})
    return multi_lang_text_list


def search_ele_mid(flaglist, tf = 'v'):
    nowidx = -1
    halflen = (len(flaglist))//2
    for gap in range(len(flaglist)-halflen):
        nowidx = halflen - gap
        if flaglist[nowidx]==tf:
            return nowidx
        nowidx = halflen + gap
        if flaglist[nowidx]==tf:
            return nowidx
    return nowidx

def add_pause(text, tf='v'):
    segment = jieba.posseg.cut(text.strip())
    wlist = []
    flist = []
    for x in segment:
        wlist.append(x.word)
        flist.append(x.flag)
    idx = search_ele_mid(flist, tf)
    if idx != len(flist)-1:
        wlist.insert(idx, sil1symbol)
    outtext = ''.join(wlist)
    return outtext

def has_punc(text):
    for char in text:
        if char in [',', '.', '!', '?', '，','。','？','！', sil1symbol]:
            return True
    return False

def text_split_lang(text, lang):
    if lang == 'ZH' or lang == 'zh':
        multi_lang_text_list = [{'lang':'zh', 'text_split': text}]
    elif lang == 'en':
        multi_lang_text_list = [{'lang':'en', 'text_split': text}]
    else:
        pretext_split =  re.split("(\[.*?\])", text, re.I|re.M)
        multi_lang_text_list = []
        pretext_split = list(filter(None, pretext_split))
        for utext in pretext_split:
            if utext[0] != '[':
                pattern = r'([a-zA-Z ,.\!\?]+|[\u4e00-\u9fa5 ，。,.\t \"\！\？\“\”\、]+)'
                text_split = re.findall(pattern, utext)
                print(text_split)
                for idx in range(len(text_split)):
                    tmpts = text_split[idx]
                    tmp_lang = langid.classify(tmpts)[0]
                    if len(tmpts)>20:
                        if not has_punc(tmpts[:-1]):
                            tmpts = add_pause(tmpts, 'p')
                        if not has_punc(tmpts[:-1]):
                            tmpts = add_pause(tmpts, 'v')   
                    if tmpts in special_phrase:
                        tmpts = tmpts+sil1symbol
                    if tmp_lang in ['zh', 'jp', 'ja']:
                        tmp_lang = 'zh'
                        tmpts = tmpts.replace(' ', sil1symbol)
                    else:
                        tmp_lang = 'en' 
                    if not tmpts.isspace():
                        multi_lang_text_list.append({'lang':tmp_lang, 'text_split': tmpts})
            else:
                phones = utext[1:-1]
                multi_lang_text_list.append({'lang':'phone', 'text_split': phones})
    return multi_lang_text_list

def text_norm(text, lang):
    outtext = norm_func_dict[lang](text)
    return outtext

def g2p(text, lang):
    phoneme_list = g2p_func_dict[lang](text)
    return phoneme_list

def tokenizer(phoneme_list):
    #token_list = [all_phone_dict[pho] for pho in phoneme_list]
    token_list = [all_phone_dict[pho] if pho in all_phone_dict.keys() else all_phone_dict['sil'] for pho in phoneme_list]
    return token_list 

def postprocess(phonelist):
    outlist = [xx if xx not in punctuation else 'sil' for xx in phonelist]
    return outlist

def postprocess_tts(phonelist):
    #outlist = ['sil', '<sp>']
    outlist = []
    print(phonelist)
    for idx in range(len(phonelist)):
        pm = phonelist[idx]
        if pm not in punctuation:
            outlist.append(pm)
        elif pm == sil1symbol:
            outlist.append('sil_1')
        else:
            #outlist.append('sil')
            outlist.append('sil_punc')
            #outlist.append('<sp>')
    #if outlist[-1] == 'sil': 
    #    outlist.append('<sp>')
    #elif outlist[-2] != 'sil':
    #    outlist.append('sil')
    #    outlist.append('<sp>')
    if phonelist[-2] not in punctuation and outlist[-1].split('_')[0] != 'sil':
        #outlist.append('sil')
        outlist.append('sil_end')
        outlist.append('<sp>')
    return outlist

def text_process_old(text, lang, spflag=True):
    multi_lang_text_list = text_split_lang(text, lang) 

    all_phone = []
    for text_split_dict in multi_lang_text_list:
        use_lang = text_split_dict['lang']
        if use_lang not in norm_func_dict.keys():
            use_lang = 'zh'
        use_text = text_split_dict['text_split']
        use_text = text_norm(use_text, use_lang) 
        phone_list = g2p(use_text, use_lang)
        #all_phone.append('sil')
        all_phone.append('sil_lang')
        all_phone.append('<sp>')
        all_phone.extend(phone_list)
    #all_phone = postprocess(all_phone)
    all_phone = postprocess_tts(all_phone)
    if not spflag:
        while '<sp>' in all_phone:
            all_phone.remove('<sp>')
    return all_phone

def text_process(text, lang, spflag=True, use_lang='zh'):
    text = preprocess_first(text, use_lang=use_lang)

    multi_lang_text_list = text_split_lang(text, lang) 

    all_phone = []
    for text_split_dict in multi_lang_text_list:
        use_lang = text_split_dict['lang']
        use_text = text_split_dict['text_split']
        if use_lang == 'phone':
            phonelist = use_text.split()
            all_phone.extend(phonelist) 
        else:
            if use_lang not in norm_func_dict.keys():
                use_lang = 'zh'
            use_text = text_norm(use_text, use_lang)
            phone_list = g2p(use_text, use_lang)
            #all_phone.append('sil')
            all_phone.append('sil_lang')
            all_phone.append('<sp>')
            all_phone.extend(phone_list)
    #all_phone = postprocess(all_phone)
    all_phone = postprocess_tts(all_phone)
    if not spflag:
        while '<sp>' in all_phone:
            all_phone.remove('<sp>')
    return all_phone

def replace_sil2label_old(phones):
    phones = ['sil_1' if xx == 'sil_lang' else xx for xx in phones]
    phones = ['sil_2' if xx == 'sil_punc' else xx for xx in phones]
    phones = ['sil_2' if xx == 'sil_end' else xx for xx in phones]
    phones = ['sil_1' if xx == 'sil' else xx for xx in phones]
    outphones = []
    for ele in phones:
        if outphones == []:
            outphones.append(ele)
        else:
            if ele.split('_')[0] == 'sil' and outphones[-1].split('_')[0] == 'sil':
                #outphones[-1] = 'sil_2'
                outphones[-1] = 'sil_1'
            else:
                outphones.append(ele)
    if outphones[-1].split('_')[0] == 'sil':
        outphones = outphones[:-1]
    return outphones

def replace_sil2label_0808(phones):
    #phones = ['sil_1' if xx == 'sil_lang' else xx for xx in phones]
    phones = ['' if xx == 'sil_lang' else xx for xx in phones]
    phones = ['sil_2' if xx == 'sil_punc' else xx for xx in phones]
    phones = ['sil_2' if xx == 'sil_end' else xx for xx in phones]
    phones = ['sil_1' if xx == 'sil' else xx for xx in phones]
    phones = list(filter(None, phones))
    #outphones = []
    outphones = ['sil_1']
    for ele in phones:
        if outphones == []:
            outphones.append(ele)
        else:
            if ele.split('_')[0] == 'sil' and outphones[-1].split('_')[0] == 'sil':
                #outphones[-1] = 'sil_2'
                outphones[-1] = 'sil_1'
            else:
                outphones.append(ele)
    if outphones[-1].split('_')[0] == 'sil':
        outphones = outphones[:-1]
    return outphones

def replace_sil2label(phones):
    #phones = ['sil_1' if xx == 'sil_lang' else xx for xx in phones]
    phones = ['' if xx == 'sil_lang' else xx for xx in phones]
    phones = ['sil_2' if xx == 'sil_punc' else xx for xx in phones]
    phones = ['sil_2' if xx == 'sil_end' else xx for xx in phones]
    phones = ['sil_1' if xx == 'sil' else xx for xx in phones]
    phones = list(filter(None, phones))
    #outphones = []
    outphones = ['sil_1']
    for ele in phones:
        if outphones == []:
            outphones.append(ele)
        else:
            if ele.split('_')[0] == 'sil' and outphones[-1].split('_')[0] == 'sil':
                outphones[-1] = 'sil_2'
                #outphones[-1] = 'sil_1'
            else:
                outphones.append(ele)
    #if outphones[-1].split('_')[0] == 'sil':
    #    outphones = outphones[:-1]
    return outphones


def text_process_asr(text, lang):
    multi_lang_text_list = text_split_lang(text, lang) 

    all_phone = []
    for text_split_dict in multi_lang_text_list:
        use_lang = text_split_dict['lang']
        use_text = text_split_dict['text_split']
        use_text = text_norm(use_text, use_lang) 
        phone_list = g2p(use_text, use_lang)
        phone_list_new = []
        for idx in range(len(phone_list)):
            tmpp = phone_list[idx]
            if tmpp != '<sp>':
                phone_list_new.append(tmpp)
        all_phone.extend(phone_list_new)
    all_phone = postprocess(all_phone)
    if all_phone[0] != 'sil':
        all_phone = ['sil'] + all_phone 
    if all_phone[-1] != 'sil': 
        all_phone = all_phone + ['sil']
    
    return all_phone


'''
FilePath: /python-Sigilyph/sigilyph/__init__.py
Descripttion: 
Author: Yixiang Chen
version: 
Date: 2025-05-13 11:01:07
LastEditors: Yixiang Chen
LastEditTime: 2025-09-25 17:22:30
'''

from sigilyph.core.sigilyph_class import Sigilyph

from sigilyph.core.symbols import all_phone_dict

from sigilyph.core.bert_align import AlignBert




'''
FilePath: /python-Sigilyph/setup.py
Descripttion: 
Author: Yixiang Chen
version: 
Date: 2025-03-24 15:57:41
LastEditors: Yixiang Chen
LastEditTime: 2025-09-26 15:51:52
'''

from setuptools import setup, find_packages

VERSION = '0.3.1' 
DESCRIPTION = 'Text Front for TTS'
#LONG_DESCRIPTION = 'Data Package for TTS with a slightly longer description'
LONG_DESCRIPTION = open("README.md", encoding="utf-8").read()

# 配置
setup(
        name="sigilyph", 
        version=VERSION,
        author="Yixiang Chen",
        author_email="<yixiangchen1995@gmail.com>",
        license='MIT',
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        long_description_content_type="text/markdown",
        url="https://github.com/yixiangchen1995/python-Sigilyph",
        packages=(
            find_packages()
        ),
        include_package_data=True,
        install_requires=[
            'g2p_en',
            'jieba',
            'jieba_fast',
            'pypinyin',
            'WeTextProcessing==1.0.3',
            'langid'
        ], # add any additional packages that ## add tinytag package
        
        keywords=['python', 'first package'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Developers",
            'Programming Language :: Python',
            'Programming Language :: Python :: 3.10',
        ]
)


'''
FilePath: /python-Sigilyph/sigilyph/core/sigilyph_class.py
Descripttion: 
Author: Yixiang Chen
version: 
Date: 2025-08-12 14:42:50
LastEditors: Yixiang Chen
LastEditTime: 2025-08-12 19:36:22
'''

import langid
import re
import json

import jieba
import jieba.posseg

from sigilyph.core.g2p_func import g2p_en, g2p_cn
from sigilyph.core.norm_func import preprocess_first, text_norm_en, text_norm_cn
from sigilyph.core.symbols import punctuation 
from sigilyph.core.predict import before_replace_dict, special_word_dict, special_phrase

norm_func_dict = {
    'en': text_norm_en,
    'zh': text_norm_cn
}

g2p_func_dict = {
    'en': g2p_en,
    'zh': g2p_cn
}

class Sigilyph:
    def __init__(self, before_dict_path=None, special_dict_path=None):
        self.sil1symbol='-'
        self.punctuation = punctuation

        self.before_replace_dict = before_replace_dict 
        if before_dict_path:
            with open(before_dict_path, 'r', encoding="utf-8") as obdp:
                extra_before_dict = json.load(obdp)
            self.before_replace_dict.update(extra_before_dict)

        self.special_word_dict = special_word_dict
        if special_dict_path:
            with open(special_dict_path, 'r', encoding="utf-8") as obdp:
                extra_special_dict = json.load(obdp)
            self.special_word_dict.update(extra_special_dict)
        
        self.special_phrase = special_phrase 
    
    def forward(self, text, lang, spflag=False, norm_use_lang='zh'):
        phones = self.text_process(text, lang, spflag, norm_use_lang)
        phones = self.replace_sil2label(phones)
        return phones

    def text_process(self, text, lang, spflag=False, norm_use_lang='zh'):
        text = preprocess_first(text, self.before_replace_dict, special_word_dict, norm_use_lang=norm_use_lang)

        multi_lang_text_list = self.text_split_lang(text, lang) 

        all_phone = []
        for text_split_dict in multi_lang_text_list:
            use_lang = text_split_dict['lang']
            use_text = text_split_dict['text_split']
            if use_lang == 'phone':
                phonelist = use_text.split()
                all_phone.extend(phonelist) 
            else:
                if use_lang not in norm_func_dict.keys():
                    use_lang = 'zh'
                use_text = self.text_norm(use_text, use_lang)
                phone_list = self.g2p(use_text, use_lang)
                #all_phone.append('sil')
                all_phone.append('sil_lang')
                all_phone.append('<sp>')
                all_phone.extend(phone_list)
        #all_phone = postprocess(all_phone)
        all_phone = self.postprocess_tts(all_phone)
        if not spflag:
            while '<sp>' in all_phone:
                all_phone.remove('<sp>')
        return all_phone

    ###############  split text in line with lang ##############
    def text_split_lang(self, text, lang):
        if lang == 'ZH' or lang == 'zh':
            multi_lang_text_list = [{'lang':'zh', 'text_split': text}]
        elif lang == 'en':
            multi_lang_text_list = [{'lang':'en', 'text_split': text}]
        else:
            pretext_split =  re.split("(\[.*?\])", text, re.I|re.M)
            multi_lang_text_list = []
            pretext_split = list(filter(None, pretext_split))
            for utext in pretext_split:
                if utext[0] != '[':
                    pattern = r'([a-zA-Z ,.\!\?]+|[\u4e00-\u9fa5 ，。,.\t \"\！\？\“\”\、]+)'
                    text_split = re.findall(pattern, utext)
                    #print(text_split)
                    for idx in range(len(text_split)):
                        tmpts = text_split[idx]
                        tmp_lang = langid.classify(tmpts)[0]
                        if len(tmpts)>20:
                            if not self.has_punc(tmpts[:-1]):
                                tmpts = self.add_pause(tmpts, 'p')
                            if not self.has_punc(tmpts[:-1]):
                                tmpts = self.add_pause(tmpts, 'v')   
                        if tmpts in self.special_phrase:
                            tmpts = tmpts+self.sil1symbol
                        if tmp_lang in ['zh', 'jp', 'ja']:
                            tmp_lang = 'zh'
                            tmpts = tmpts.replace(' ', self.sil1symbol)
                        else:
                            tmp_lang = 'en' 
                        if not tmpts.isspace():
                            multi_lang_text_list.append({'lang':tmp_lang, 'text_split': tmpts})
                else:
                    phones = utext[1:-1]
                    multi_lang_text_list.append({'lang':'phone', 'text_split': phones})
        return multi_lang_text_list
    
    ##########  add parse ###############
    def has_punc(self, text):
        for char in text:
            if char in [',', '.', '!', '?', '，','。','？','！', self.sil1symbol]:
                return True
        return False
    
    def add_pause(self, text, tf='v'):
        segment = jieba.posseg.cut(text.strip())
        wlist = []
        flist = []
        for x in segment:
            wlist.append(x.word)
            flist.append(x.flag)
        idx = self.search_ele_mid(flist, tf)
        if idx != len(flist)-1:
            wlist.insert(idx, self.sil1symbol)
        outtext = ''.join(wlist)
        return outtext
    
    def search_ele_mid(self, flaglist, tf = 'v'):
        nowidx = -1
        halflen = (len(flaglist))//2
        for gap in range(len(flaglist)-halflen):
            nowidx = halflen - gap
            if flaglist[nowidx]==tf:
                return nowidx
            nowidx = halflen + gap
            if flaglist[nowidx]==tf:
                return nowidx
        return nowidx

    ######## text norm #########
    def text_norm(self, text, lang):
        outtext = norm_func_dict[lang](text)
        return outtext

    ############ g2p ################
    def g2p(self, text, lang):
        phoneme_list = g2p_func_dict[lang](text)
        return phoneme_list
    
    ############# post process #############
    def postprocess_tts(self, phonelist):
        #outlist = ['sil', '<sp>']
        outlist = []
        #print(phonelist)
        for idx in range(len(phonelist)):
            pm = phonelist[idx]
            if pm not in self.punctuation:
                outlist.append(pm)
            elif pm == self.sil1symbol:
                outlist.append('sil_1')
            else:
                #outlist.append('sil')
                outlist.append('sil_punc')
                #outlist.append('<sp>')
        #if outlist[-1] == 'sil': 
        #    outlist.append('<sp>')
        #elif outlist[-2] != 'sil':
        #    outlist.append('sil')
        #    outlist.append('<sp>')
        if phonelist[-2] not in self.punctuation and outlist[-1].split('_')[0] != 'sil':
            #outlist.append('sil')
            outlist.append('sil_end')
            outlist.append('<sp>')
        return outlist

    ########## replace silence token ###############
    def replace_sil2label(self, phones):
        #phones = ['sil_1' if xx == 'sil_lang' else xx for xx in phones]
        phones = ['' if xx == 'sil_lang' else xx for xx in phones]
        phones = ['sil_2' if xx == 'sil_punc' else xx for xx in phones]
        phones = ['sil_2' if xx == 'sil_end' else xx for xx in phones]
        phones = ['sil_1' if xx == 'sil' else xx for xx in phones]
        phones = list(filter(None, phones))
        #outphones = []
        outphones = ['sil_1']
        for ele in phones:
            if outphones == []:
                outphones.append(ele)
            else:
                if ele.split('_')[0] == 'sil' and outphones[-1].split('_')[0] == 'sil':
                    outphones[-1] = 'sil_2'
                    #outphones[-1] = 'sil_1'
                else:
                    outphones.append(ele)
        #if outphones[-1].split('_')[0] == 'sil':
        #    outphones = outphones[:-1]
        return outphones
    
    
    




'''
FilePath: /python-Sigilyph/sigilyph/core/norm_func.py
Descripttion: 
Author: Yixiang Chen
version: 
Date: 2025-03-31 17:50:26
LastEditors: Yixiang Chen
LastEditTime: 2025-09-26 14:45:16
'''


import re

from sigilyph.core.symbols import punctuation, punc_map_ch

from tn.chinese.normalizer import Normalizer as ZhNormalizer
from tn.english.normalizer import Normalizer as EnNormalizer


import os
from importlib_resources import files
basedir = files('sigilyph')

#zh_tn_model = ZhNormalizer(remove_erhua=False, full_to_half=False)
#en_tn_model = EnNormalizer()
#zh_tn_model = ZhNormalizer(cache_dir='./sigilyph/core/cache_dir', remove_erhua=False, full_to_half=False)
#en_tn_model = EnNormalizer(cache_dir='./sigilyph/core/cache_dir')
zh_tn_model = ZhNormalizer(cache_dir=os.path.join(basedir, 'core', 'cache_dir'), remove_erhua=False, full_to_half=False)
en_tn_model = EnNormalizer(cache_dir=os.path.join(basedir, 'core', 'cache_dir'))

import json
#import sys
#sys.path.append('text_front')
#with open('./special_dict.json', 'r', encoding="utf-8") as infi:
#with open('./text_front/special_dict.json', 'r', encoding="utf-8") as infi:
#with open('./sigilyph/core/special_dict.json', 'r', encoding="utf-8") as infi:
with open(os.path.join(basedir, 'core', 'special_dict.json'), 'r', encoding="utf-8") as infi:
    special_dict = json.load(infi)

def pro_norm(text, use_lang='zh'):
    if use_lang == 'zh':
        norm_text = zh_tn_model.normalize(text)
        #print("zh ", norm_text)
    else:
        norm_text = en_tn_model.normalize(text)
        #print("en ", norm_text)
    return norm_text

def replace_with_dict(text, replace_dict):
    for old, new in replace_dict.items():
        text = text.replace(old, new)
    return text
def replace_with_dict_re(text, replace_dict):
    pattern = re.compile("|".join(re.escape(key) for key in replace_dict.keys()))
    return pattern.sub(lambda m: replace_dict[m.group(0)], text)

pre_replace_dict = {"AlphaFold-Plus": "AlphaFold Plus"}
def preprocess_first_old(text, use_lang='zh'):
    text = replace_with_dict(text, pre_replace_dict)
    norm_text = pro_norm(text, use_lang)
    #print(norm_text)
    rep_text = replace_with_dict(norm_text, special_dict)
    return rep_text

def preprocess_first(text, before_replace_dict, special_word_dict, norm_use_lang='zh'):
    text = replace_with_dict(text, before_replace_dict)
    norm_text = pro_norm(text, norm_use_lang)
    #print(norm_text)
    rep_text = replace_with_dict(norm_text, special_word_dict)
    return rep_text


def normalizer(text):
    return text

def replace_punc(text):
    #text = text.replace("嗯", "恩").replace("呣", "母")
    pattern = re.compile("|".join(re.escape(p) for p in punc_map_ch.keys()))
    replaced_text = pattern.sub(lambda x: punc_map_ch[x.group()], text)
    replaced_text = re.sub(
        r"[^\u4e00-\u9fa5" + "".join(punctuation) + r"]+", "", replaced_text
    )
    return replaced_text

def text_norm_cn(text):
    text = normalizer(text)
    text = replace_punc(text)
    return text

def text_norm_en(text):
    
    return text 